package com.ruoyi.web.controller.app;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.aliyuncs.dysmsapi.model.v20170525.SendSmsResponse;
import com.aliyuncs.exceptions.ClientException;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.AsArticleClassify;
import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.CommonUtil;
import com.ruoyi.common.utils.SendAliSmsUtil;
import com.ruoyi.common.utils.SendSmsVo;
import com.ruoyi.common.utils.map.GeoUtils;
import com.ruoyi.system.domain.*;
import com.ruoyi.system.mapper.AsUserMapper;
import com.ruoyi.system.service.*;
import com.wechat.pay.java.service.payments.model.Transaction;
import com.wechat.pay.java.service.refund.model.Refund;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

/**
 * app接口（不需要登录的）
 *  校验
 * @author ruoyi
 */
@RestController
@RequestMapping("/app")
public class AppController extends BaseController
{
    @Autowired
    private IAsArticleService asArticleService;

    @Autowired
    private IEtOperatingAreaService etOperatingAreaService;

    @Autowired
    private IEtParkingAreaService etParkingAreaService;

    @Autowired
    private IAsDeviceService asDeviceService;

    @Autowired
    private IEtFeeRuleService etFeeRuleService;

    @Autowired
    private IEtOrderService etOrderService;

    @Autowired
    private IWxPayService wxPayService;

    @Autowired
    private IAsArticleClassifyService asArticleClassifyService;

    @Resource
    private AsUserMapper asUserMapper;

    @Value("${aliyun.accessKeyId}")
    private String accessKeyId;

    @Value("${aliyun.accessKeySecret}")
    private String accessKeySecret;

    @Value("${aliyun.signName}")
    private String signName;

    @Value("${aliyun.templateCode}")
    private String templateCode;


    /**
     * 根据查询文章分类列表
     */
    @GetMapping("/classify/list")
    public AjaxResult classifyList(AsArticleClassify asArticleClassify)
    {
        logger.info("查询分类列表：【asArticleClassify="+asArticleClassify+"】");
        List<AsArticleClassify> asArticleClassifies = asArticleClassifyService.selectClassifyList(asArticleClassify);
        return success(asArticleClassifies);
    }

    /**
     * 查询文章列表
     */
    @GetMapping("/article/list")
    public TableDataInfo articleList(EtArticle etArticle)
    {
        startPage();
        List<EtArticle> list = asArticleService.selectAsArticleListByApp(etArticle);
        return getDataTable(list);
    }

    /**
     * 获取文章详细信息
     */
    @GetMapping(value = "/article/{articleId}")
    public AjaxResult getArticleInfo(@PathVariable("articleId") Long articleId)
    {
        return success(asArticleService.selectAsArticleByArticleId(articleId));
    }


    /**
     * 查询运营区列表
     */
    @GetMapping("/area/list")
    public TableDataInfo list(EtOperatingArea etOperatingArea)
    {
        etOperatingArea.setStatus("0");//运营区状态：0-运营中；1-停运
        startPage();
        List<EtOperatingArea> list = etOperatingAreaService.selectEtOperatingAreaList(etOperatingArea);
        return getDataTable(list);
    }

    /**
     * 获取运营区详细信息
     */
    @GetMapping(value = "/area/{areaId}")
    public AjaxResult getInfo(@PathVariable("areaId") Long areaId)
    {
        if(areaId == null){
            logger.info("没有传入areaId参数：【areaId={}】",areaId);
            return error("请传areaId参数"+"【areaId="+areaId+"】");
        }
        EtOperatingArea area = etOperatingAreaService.selectEtOperatingAreaByAreaId(areaId);
        return success(area);
    }


    /**
     * 根据定位获取运营区详细信息
     */
    @GetMapping(value = "/area/info")
    public AjaxResult areaInfo(String longitude,String latitude, String deptId,String sn)
    {
        if(StrUtil.isBlank(longitude) || StrUtil.isBlank(latitude)){
            logger.info("没有经纬度参数：【longitude={}】，【latitude={}】",longitude,latitude);
            return error("请传经纬度参数"+"【longitude="+longitude+"】，【latitude="+latitude+"】");
        }
        EtOperatingArea area = etOperatingAreaService.getAreaInfoByLocation(longitude,latitude,deptId,sn);
        return success(area);
    }

    /**
     * 查询停车区列表
     */
    @GetMapping("/parking/list")
    public TableDataInfo list(EtParkingArea etParkingArea)
    {
        etParkingArea.setStatus("0");
//        startPage();
        List<EtParkingArea> list = etParkingAreaService.selectEtParkingAreaList(etParkingArea);
        return getDataTable(list);
    }

    /**
     * 获取停车区详细信息
     */
    @GetMapping(value = "/parking/{parkingId}")
    public AjaxResult getParkingInfo(@PathVariable("parkingId") Long parkingId)
    {
        return success(etParkingAreaService.selectEtParkingAreaByParkingId(parkingId));
    }


//    /**
//     * 根据定位获取运营区信息，并返回所有车辆定位
//     */
//    @GetMapping(value = "/vehicleLocalization")
//    public AjaxResult vehicleLocalization(String longitude,String latitude)
//    {
//        if(StrUtil.isBlank(longitude) || StrUtil.isBlank(latitude)){
//            logger.info("没有经纬度参数：【longitude={}】，【latitude={}】",longitude,latitude);
//            return error("请传经纬度参数"+"【longitude="+longitude+"】，【latitude="+latitude+"】");
//        }
////        webSocket.SendMessage("需要发送的消息", "识别唯一session");
//        List<AsDevice> asDevices = asDeviceService.vehicleLocalization(longitude,latitude);
//        return success(asDevices);
//    }

    /**
     * 根据运营区id获取所有车辆
     */
    @GetMapping(value = "/allVehicleByArea")
    public AjaxResult allVehicleByArea(Long areaId)
    {
        if(areaId == null){
            logger.info("没有传入areaId参数：【areaId={}】",areaId);
            return error("请传areaId参数"+"【areaId="+areaId+"】");
        }
        List<AsDevice> asDevices = asDeviceService.allVehicleByArea(areaId);
        return success(asDevices);
    }


    /**
     * 根据运营区获取收费方式列表
     */
    @GetMapping("/fee/list")
    public AjaxResult feeList(EtFeeRule etFeeRule)
    {
        List<EtFeeRule> list = etFeeRuleService.selectEtFeeRuleListByAreaId(etFeeRule.getAreaId());
        return success(list);
    }

    /**
     * 获取计费套餐详细信息
     */
    @GetMapping(value = "/fee/{ruleId}")
    public AjaxResult getFeeInfo(@PathVariable("ruleId") Long ruleId)
    {
        return success(etFeeRuleService.selectEtFeeRuleByRuleId(ruleId));
    }

//    /**
//     * 获取计费规则  多少分钟内  里程数
//     */
//    @GetMapping(value = "/fee/rule")
//    public AjaxResult getRule()
//    {
//        String appointmentServiceFee = sysConfigService.selectConfigByKey("appointment.service.fee");//预约服务费
//        String dispatchFee = sysConfigService.selectConfigByKey("dispatch.fee");//调度费
//        String vehicleManagementFee = sysConfigService.selectConfigByKey("vehicle.management.fee");//车辆停车点外调度费
//        String startingPrice = sysConfigService.selectConfigByKey("starting.price");//起步价
//        String startingHowManyMinutes = sysConfigService.selectConfigByKey("starting.how.many.minutes");//多少分钟内按起步价
//        String timeFee = sysConfigService.selectConfigByKey("time.fee");//时长费
//        String timeMinutes = sysConfigService.selectConfigByKey("time.minutes");//时长分钟
//        String timeoutMinutes = sysConfigService.selectConfigByKey("timeout.minutes");//预约超时保留分钟
//        String deposit = sysConfigService.selectConfigByKey("deposit");//押金
//        String isMeal = sysConfigService.selectConfigByKey("is.meal");//是否有套餐 是否有套餐：0-没有套餐；1-有套餐
//        String cappingFee = sysConfigService.selectConfigByKey("order.capping.fee");//24小时内封顶费用
//        String forceParkingArea = sysConfigService.selectConfigByKey("force.parking.area");//是否强制在停车区内还车
//        FeeRuleVo build = FeeRuleVo.builder()
//                .appointmentServiceFee(appointmentServiceFee)
//                .dispatchFee(dispatchFee)
//                .vehicleManagementFee(vehicleManagementFee)
//                .startingPrice(startingPrice)
//                .startingHowManyMinutes(startingHowManyMinutes)
//                .timeFee(timeFee)
//                .timeMinutes(timeMinutes)
//                .timeoutMinutes(timeoutMinutes)
//                .deposit(deposit)
//                .isMeal(isMeal)
//                .cappingFee(cappingFee)
//                .forceParkingArea(forceParkingArea)
//                .build();
//        return success(build);
//    }


    /**
     * 响铃寻车
     */
    @PostMapping("/device/ring")
    public AjaxResult ring(String sn)
    {
        if(StrUtil.isBlank(sn)){
            logger.info("没有sn号参数：【sn={}】",sn);
            return error("请传sn号参数"+"【sn="+sn+"】");
        }
       Boolean i =asDeviceService.ring(sn);
        return success(i);
    }

    /**
     * 响铃寻车用mac
     */
    @PostMapping("/device/ringByMac")
    public AjaxResult ringByMac(String mac)
    {
        if(StrUtil.isBlank(mac)){
            logger.info("没有mac号参数：【mac={}】",mac);
            return error("请传mac号参数"+"【mac="+mac+"】");
        }
        Boolean i =asDeviceService.ringByMac(mac);
        return success(i);
    }


    /**
     * 根据sn号查询车辆实时信息
     */
    @GetMapping("/device/info")
    public AjaxResult deviceInfo(String sn)
    {
        if(StrUtil.isBlank(sn)){
            logger.info("没有sn号参数：【sn={}】",sn);
            return error("请传sn号参数"+"【sn="+sn+"】");
        }
        AsDevice device = asDeviceService.selectAsDeviceBySn(sn);
        return success(device);
    }

    /**
     * 查询当前用户是否有正在进行中的订单
     */
    @PostMapping("/user/isInOrder")
    public AjaxResult isInOrder(Long userId)
    {
        if(StrUtil.isBlank(userId.toString())){
            logger.info("没有userId参数：【userId={}】",userId);
            return error("请传userId参数"+"【userId="+userId+"】");
        }
        List<EtOrder> orders = etOrderService.isInOrder(userId,null);
        return success(orders);
    }


    /**
     * 根据定位获取地址    逆地理编码
     */
    @GetMapping("/getAddress")
    public AjaxResult deviceList(String location)
    {
//        logger.info("根据定位获取地址：【location="+location+"】");
//        String address = CommonUtil.getAddressByGeo(location);
//        logger.info("根据定位获取地址：【address="+address+"】");
        return AjaxResult.success();
    }

    /**
     * 获取省市县
     */
    @GetMapping("/getDistrictList")
    public AjaxResult getDistrictList()
    {
        logger.info("【获取省市县】");
        String address = CommonUtil.getDistrictList();
//        logger.info("【获取省市县】【address="+address+"】");
        return AjaxResult.success("操作成功", address);
    }

    /**
     * 查询退款是否成功
     */
    @GetMapping("/queryByOutRefundNo")
    public AjaxResult queryByOutRefundNo(Long areaId,String outRefundNo)
    {
        logger.info("查询退款是否成功：【outRefundNo="+outRefundNo+"】");
        Refund refund = wxPayService.queryByOutRefundNo(areaId,outRefundNo);
        return AjaxResult.success(refund);
    }

    /**
     * 通过商户订单号查询订单信息
     */
    @GetMapping("/queryOrderByOutTradeNo")
    public AjaxResult queryOrderByOutTradeNo(String outTradeNo)
    {
        logger.info("查询支付订单信息：【outTradeNo="+outTradeNo+"】");
        Transaction transaction = wxPayService.queryOrderByOutTradeNo(outTradeNo);
        return AjaxResult.success(transaction);
    }

    /**
     * 删除订单
     */
    @DeleteMapping("/{orderNo}")
    public AjaxResult remove(@PathVariable String orderNo)
    {
        return toAjax(etOrderService.deleteEtOrderByOrderNo(orderNo));
    }

    /**
     * 根据mac号判断是否有绑定过
     */
    @GetMapping("/isBandByMac")
    public AjaxResult isBandByMac(String mac)
    {
        AjaxResult ajax = AjaxResult.success();
        logger.info("根据mac号判断是否有绑定过：【mac="+mac+"】");
        if(StrUtil.isBlank(mac)){
            throw new ServiceException("mac号不能为空");
        }
        ajax.put(AjaxResult.DATA_TAG,asDeviceService.isBandByMac(mac));
        return ajax;
    }

    /**
     * 计算订单距离
     */
    @GetMapping("/calculateDistance")
    public AjaxResult calculateDistance()
    {
        EtOrder order = new EtOrder();
        order.setType("1");
        List<EtOrder> orders = etOrderService.selectEtOrderList(order);
        for(EtOrder etOrder:orders){
            String tripRouteStr = etOrder.getTripRouteStr();
            if(StrUtil.isNotBlank(tripRouteStr)){
                double[][] doubles = GeoUtils.parseJsonTrack(tripRouteStr);
                double v = GeoUtils.calculateTotalDistance(doubles);
                etOrder.setDistance((int)Math.round(v));
                int updateEtOrder = etOrderService.updateEtOrder(etOrder);
                if(updateEtOrder>0){
                    logger.info("计算订单距离成功：【orderNo="+etOrder.getOrderNo()+"】");
                }
            }
        }
        return AjaxResult.success();
    }

    /**
     * 查询版本并更新
     */
    @GetMapping("/queryVersion")
    public AjaxResult queryVersion(String sn)
    {
        if(StrUtil.isNotBlank(sn)){
            return AjaxResult.success(asDeviceService.updateVersion(sn));
        }
        List<AsDevice> asDevices = asDeviceService.selectAsDeviceList(new AsDevice());
        for (AsDevice asDevice:asDevices) {
            asDeviceService.updateVersion(asDevice.getSn());
        }
        return AjaxResult.success();
    }
    @GetMapping("/sendMsg")
    public AjaxResult getUserInfo(String orderNo)
    {
        EtOrder order = etOrderService.selectEtOrderByOrderNo(orderNo);
        /** 发送一个短信给运营商*/
        SysDept sysDept = wxPayService.getDeptObjByAreaId(order.getAreaId());
        logger.info("【微信支付回调】还车拍照审核，获取到运营商：【{}】", JSON.toJSON(sysDept));
        if(ObjectUtil.isNull(sysDept.getAppUserId())){
            throw new ServiceException("该运营商【"+sysDept.getDeptName()+"】未绑定微信用户");
        }
        AsUser asUser1 = asUserMapper.selectUserById(sysDept.getAppUserId());
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("name",order.getPayFee());
        String phone = asUser1.getPhonenumber();
        SendSmsVo sendSmsVo = new SendSmsVo();
        sendSmsVo.setMobile(phone);
        sendSmsVo.setTemplateCode(templateCode);
        sendSmsVo.setParam(jsonObject.toJSONString());
        sendSmsVo.setSignName(signName);
        SendSmsResponse response = null;
        logger.info("【微信支付回调】向阿里云发送短信，请求,----------【{}】", JSON.toJSONString(sendSmsVo));
        try {
            response = SendAliSmsUtil.sendVerifyCode(accessKeyId,accessKeySecret,sendSmsVo);
        } catch (ClientException e) {
            e.printStackTrace();
        }
        logger.info("【微信支付回调】发送阿里云短信成功,返回----------【{}】",JSON.toJSONString(response));
        return AjaxResult.success();
    }

}
