package com.ruoyi.web.controller.system;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.constant.UserConstants;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.system.domain.EtAreaDept;
import com.ruoyi.system.mapper.EtAreaDeptMapper;
import com.ruoyi.system.mapper.EtOperatingAreaMapper;
import com.ruoyi.system.mapper.SysUserMapper;
import com.ruoyi.system.service.IEtOperatingAreaService;
import com.ruoyi.system.service.ISysDeptService;
import org.apache.commons.lang3.ArrayUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

/**
 * 运营商信息
 *
 * @author ruoyi
 */
@RestController
@RequestMapping("/system/dept")
public class SysDeptController extends BaseController
{
    @Autowired
    private ISysDeptService deptService;

    @Autowired
    private IEtOperatingAreaService etOperatingAreaService;

    @Resource
    private EtAreaDeptMapper etAreaDeptMapper;

    @Resource
    private EtOperatingAreaMapper etOperatingAreaMapper;


    /**
     * 获取运营商列表
     */
    @PreAuthorize("@ss.hasPermi('system:dept:list')")
    @GetMapping("/list")
    public AjaxResult list(SysDept dept)
    {
        startPage();
        List<SysDept> depts = deptService.selectDeptList(dept);
        return success(depts);
    }

    /**
     * 查询运营商列表（排除节点）
     */
    @PreAuthorize("@ss.hasPermi('system:dept:list')")
    @GetMapping("/list/exclude/{deptId}")
    public AjaxResult excludeChild(@PathVariable(value = "deptId", required = false) Long deptId)
    {
        List<SysDept> depts = deptService.selectDeptList(new SysDept());
        depts.removeIf(d -> d.getDeptId().intValue() == deptId || ArrayUtils.contains(StringUtils.split(d.getAncestors(), ","), deptId + ""));
        return success(depts);
    }

    /**
     * 根据运营商编号获取详细信息
     */
    @PreAuthorize("@ss.hasPermi('system:dept:query')")
    @GetMapping(value = "/{deptId}")
    public AjaxResult getInfo(@PathVariable Long deptId)
    {
        AjaxResult ajax = AjaxResult.success();
        deptService.checkDeptDataScope(deptId);
        ajax.put(AjaxResult.DATA_TAG,deptService.selectDeptById(deptId));
        ajax.put("areaIds", etOperatingAreaService.selectAreaListByDeptId(deptId));
        return ajax;
    }

    /**
     * 新增运营商
     */
    @PreAuthorize("@ss.hasPermi('system:dept:add')")
    @Log(title = "运营商管理", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@Validated @RequestBody SysDept dept)
    {
        if (!deptService.checkDeptNameUnique(dept))
        {
            return error("新增运营商'" + dept.getDeptName() + "'失败，运营商名称已存在");
        }
        Long[] areaIds = dept.getAreaIds();
        if (ObjectUtil.isNotNull(areaIds))
        {
            for (Long areaId:areaIds) {
                List<EtAreaDept> areaId1 = etAreaDeptMapper.selectList(new QueryWrapper<EtAreaDept>().eq("area_id", areaId));
                if (ObjectUtil.isNotEmpty(areaId1)){
                    return error("运营区'" + etOperatingAreaMapper.selectById(areaId1.get(0)).getAreaName() + "'已经被绑定");
                }
            }
        }
        dept.setCreateBy(getUsername());
        return toAjax(deptService.insertDept(dept));
    }

    /**
     * 修改运营商
     */
    @PreAuthorize("@ss.hasPermi('system:dept:edit')")
    @Log(title = "运营商管理", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@Validated @RequestBody SysDept dept)
    {
        Long deptId = dept.getDeptId();
        deptService.checkDeptDataScope(deptId);
        if (!deptService.checkDeptNameUnique(dept))
        {
            return error("修改运营商'" + dept.getDeptName() + "'失败，运营商名称已存在");
        }
        else if (dept.getParentId().equals(deptId))
        {
            return error("修改运营商'" + dept.getDeptName() + "'失败，上级运营商不能是自己");
        }
        else if (StringUtils.equals(UserConstants.DEPT_DISABLE, dept.getStatus()) && deptService.selectNormalChildrenDeptById(deptId) > 0)
        {
            return error("该运营商包含未停用的子运营商！");
        }
        Long[] areaIds = dept.getAreaIds();
        if (ObjectUtil.isNotNull(areaIds))
        {
            for (Long areaId:areaIds) {
                QueryWrapper<EtAreaDept> queryWrapper = new QueryWrapper<EtAreaDept>().eq("area_id", areaId);
                queryWrapper.ne("dept_id", deptId);
                List<EtAreaDept> areaId1 = etAreaDeptMapper.selectList(queryWrapper);
                if (ObjectUtil.isNotEmpty(areaId1)){
                    return error("运营区'" + etOperatingAreaMapper.selectById(areaId1.get(0)).getAreaName() + "'已经被绑定");
                }
            }
        }
        dept.setUpdateBy(getUsername());
        return toAjax(deptService.updateDept(dept));
    }

    /**
     * 删除运营商
     */
    @PreAuthorize("@ss.hasPermi('system:dept:remove')")
    @Log(title = "运营商管理", businessType = BusinessType.DELETE)
    @DeleteMapping("/{deptId}")
    public AjaxResult remove(@PathVariable Long deptId)
    {
        if (deptService.hasChildByDeptId(deptId))
        {
            return warn("存在下级运营商,不允许删除");
        }
        if (deptService.checkDeptExistUser(deptId))
        {
            return warn("运营商存在用户,不允许删除");
        }
        deptService.checkDeptDataScope(deptId);
        return toAjax(deptService.deleteDeptById(deptId));
    }

    /**
     * 查询余额
     */
    @GetMapping("/getBalance")
    public AjaxResult getBalance()
    {
        AjaxResult ajax = AjaxResult.success();
        Long deptId = getDeptId();
        SysDept sysDept = deptService.selectDeptById(deptId);
        ajax.put(AjaxResult.DATA_TAG, sysDept.getBalance());
        return ajax;
    }

    /**
     * 查询运营商
     */
    @GetMapping("/getDept")
    public AjaxResult getDept(Long deptId)
    {
        if(ObjectUtil.isNull(deptId)) {
            deptId = getDeptId();
        }
        AjaxResult ajax = AjaxResult.success();
        SysDept sysDept = deptService.selectDeptById(deptId);
        ajax.put(AjaxResult.DATA_TAG, sysDept);
        return ajax;
    }

    /**
     * 绑定APP用户
     */
    @Log(title = "绑定APP用户", businessType = BusinessType.UPDATE)
    @PutMapping("/bandAppUser")
    public AjaxResult bandAppUser(@RequestBody SysDept dept)
    {
        logger.info("【运营商】绑定APP用户", JSON.toJSON(dept));
        return toAjax(deptService.bandAppUser(dept));
    }

}
