package com.ruoyi.system.service.impl;

import java.util.List;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ruoyi.common.annotation.DataScope;
import com.ruoyi.common.constant.IotConstants;
import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.CommonUtil;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.onenet.ResponseVo;
import com.ruoyi.common.utils.onenet.Token;
import com.ruoyi.system.domain.AsDevice;
import com.ruoyi.system.domain.EtOperatingArea;
import com.ruoyi.system.mapper.AsDeviceMapper;
import com.ruoyi.system.service.IAsDeviceService;
import com.ruoyi.system.service.IEtOperatingAreaService;
import com.ruoyi.system.service.ISysDeptService;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.system.mapper.EtModelMapper;
import com.ruoyi.system.domain.EtModel;
import com.ruoyi.system.service.IEtModelService;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;

/**
 * 车辆型号Service业务层处理
 *
 * @author 邱贞招
 * @date 2024-05-13
 */
@Slf4j
@Service
public class EtModelServiceImpl implements IEtModelService
{
    @Resource
    private EtModelMapper etModelMapper;

    @Autowired
    private IAsDeviceService asDeviceService;

    @Autowired
    private ISysDeptService deptService;

    @Resource
    private AsDeviceMapper asDeviceMapper;

    @Resource
    private IEtOperatingAreaService etOperatingAreaService;


    /**
     * 查询车辆型号
     *
     * @param modelId 车辆型号主键
     * @return 车辆型号
     */
    @Override
    public EtModel selectEtModelByModelId(Long modelId)
    {
        EtModel etModel = etModelMapper.selectEtModelByModelId(modelId);
        if(ObjectUtil.isNotNull(etModel)){
            Integer allNum = asDeviceService.selectCountByModelId(modelId);
            etModel.setDeviceNum(allNum);
            if(ObjectUtil.isNotNull(etModel.getOperator())){
                List<EtOperatingArea> areaList = etOperatingAreaService.selectAreaListByDeptId2(etModel.getOperator());
                etModel.setAreaList(areaList);
            }
        }
        return etModel;
    }

    /**
     * 查询车辆型号列表
     *
     * @param etModel 车辆型号
     * @return 车辆型号
     */
    @Override
    @DataScope(deptAlias = "d")
    public List<EtModel> selectEtModelList(EtModel etModel)
    {
        List<EtModel> etModels = etModelMapper.selectEtModelList(etModel);
        for(EtModel model: etModels){
            // 投放数量统计
            AsDevice device = new AsDevice();
            device.setModelId(model.getModelId());
            List<AsDevice> asDevices = asDeviceService.selectAsDeviceList(device);
            model.setDeviceNum(asDevices.size());
            // 运营商名称
            SysDept sysDept = deptService.selectDeptById(model.getOperator());
            if(ObjectUtil.isNotNull(sysDept)){
               model.setOperatorName(sysDept.getDeptName());
            }
        }
        return etModels;
    }

    /**
     * 新增车辆型号
     *
     * @param etModel 车辆型号
     * @return 结果
     */
    @SneakyThrows
    @Override
    @Transactional
    public int insertEtModel(EtModel etModel)
    {
        if(etModel.getOperator() == null){
            SysUser currentUser = SecurityUtils.getLoginUser().getUser();//获取当前系统用户
            if(!currentUser.isAdmin()){
                etModel.setOperator(currentUser.getDeptId());
            }
        }
        etModel.setCreateTime(DateUtils.getNowDate());
        int i = etModelMapper.insertEtModel(etModel);
        // 发送设置低电压命令
        Integer lowBatteryReminder = etModel.getLowBatteryReminder();
        if(ObjectUtil.isNotNull(lowBatteryReminder) && lowBatteryReminder > 0){
            AsDevice device = new AsDevice();
            device.setModelId(etModel.getModelId());
            List<AsDevice> asDevices = asDeviceMapper.selectAsDeviceList(device);
            for(AsDevice asDevice: asDevices){
                // 根据百分比计算提醒电压值
                Integer quantityByPercentage = CommonUtil.getElectricQuantityByPercentage(lowBatteryReminder, etModel.getFullVoltage(), etModel.getLowVoltage());
                String lowVoltageCommand = IotConstants.COMMAND_BAT + quantityByPercentage * 10 + "@";
                log.info("发送低电压命令：" + lowVoltageCommand);
                asDeviceService.sendCommand(asDevice.getMac(), Token.getToken(), lowVoltageCommand,"发送低电压播报",null,null);
            }
        }
        return i;
    }

    /**
     * 修改车辆型号
     *
     * @param etModel 车辆型号
     * @return 结果
     */
    @SneakyThrows
    @Override
    @Transactional
    public int updateEtModel(EtModel etModel)
    {
        if(etModel.getOperator() != null){
            SysUser currentUser = SecurityUtils.getLoginUser().getUser();//获取当前系统用户
            if(!currentUser.isAdmin()){
                etModel.setOperator(currentUser.getDeptId());
            }
        }
        etModel.setUpdateTime(DateUtils.getNowDate());
        int i = etModelMapper.updateEtModel(etModel);
        // 发送设置低电压命令
        Integer lowBatteryReminder = etModel.getLowBatteryReminder();
        if(ObjectUtil.isNotNull(lowBatteryReminder) && lowBatteryReminder > 0){
            AsDevice device = new AsDevice();
            device.setModelId(etModel.getModelId());
            List<AsDevice> asDevices = asDeviceMapper.selectAsDeviceList(device);
            for(AsDevice asDevice: asDevices){
                // 根据百分比计算提醒电压值
                Integer quantityByPercentage = CommonUtil.getElectricQuantityByPercentage(lowBatteryReminder, etModel.getFullVoltage(), etModel.getLowVoltage());
                String lowVoltageCommand = IotConstants.COMMAND_BAT + quantityByPercentage * 10 + "@";
                log.info("发送低电压命令：" + lowVoltageCommand);
                ResponseVo responseVo = asDeviceService.sendCommandWithResp(asDevice.getMac(), Token.getToken(), lowVoltageCommand,"发送低电压播报",null);
                if(responseVo.getCode()!=0){
                    log.info("【还车关锁】设备【{}】远程关锁失败", asDevice.getMac());
                }
            }
        }
        return i;
    }

    /**
     * 批量删除车辆型号
     *
     * @param modelIds 需要删除的车辆型号主键
     * @return 结果
     */
    @Override
    public int deleteEtModelByModelIds(Long[] modelIds)
    {
        return etModelMapper.deleteEtModelByModelIds(modelIds);
    }

    /**
     * 删除车辆型号信息
     *
     * @param modelId 车辆型号主键
     * @return 结果
     */
    @Override
    public int deleteEtModelByModelId(Long modelId)
    {
        return etModelMapper.deleteEtModelByModelId(modelId);
    }

    @Override
    public int selectAllCount() {
        return etModelMapper.selectAllCount();
    }

}
