package com.ruoyi.system.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ruoyi.common.constant.ServiceConstants;
import com.ruoyi.common.core.domain.entity.AsUser;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.domain.model.LoginUser;
import com.ruoyi.common.core.redis.RedisLock;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.uuid.IdUtils;
import com.ruoyi.system.domain.EtAreaDept;
import com.ruoyi.system.domain.EtOrder;
import com.ruoyi.system.domain.vo.AttachVo;
import com.ruoyi.system.mapper.EtAreaDeptMapper;
import com.ruoyi.system.service.IAsUserService;
import com.ruoyi.system.service.IEtOrderService;
import com.ruoyi.system.service.ISysDeptService;
import com.ruoyi.system.service.IWxPayService;
import com.wechat.pay.java.core.Config;
import com.wechat.pay.java.core.RSAAutoCertificateConfig;
import com.wechat.pay.java.service.payments.jsapi.JsapiService;
import com.wechat.pay.java.service.payments.jsapi.JsapiServiceExtension;
import com.wechat.pay.java.service.payments.jsapi.model.*;
import com.wechat.pay.java.service.payments.model.Transaction;
import com.wechat.pay.java.service.profitsharing.ProfitsharingService;
import com.wechat.pay.java.service.profitsharing.model.*;
import com.wechat.pay.java.service.refund.RefundService;
import com.wechat.pay.java.service.refund.model.AmountReq;
import com.wechat.pay.java.service.refund.model.CreateRequest;
import com.wechat.pay.java.service.refund.model.QueryByOutRefundNoRequest;
import com.wechat.pay.java.service.refund.model.Refund;
import com.wechat.pay.java.service.transferbatch.TransferBatchService;
import com.wechat.pay.java.service.transferbatch.model.InitiateBatchTransferRequest;
import com.wechat.pay.java.service.transferbatch.model.InitiateBatchTransferResponse;
import com.wechat.pay.java.service.transferbatch.model.TransferDetailInput;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;

/**
 * 微信支付服务
 * @author 辉
 * 2024/3/11
 */
@Service
@Slf4j
public class WxPayService implements IWxPayService {

//    @Autowired
//    private JsapiService jsapiService;

//    @Autowired
//    private JsapiServiceExtension jsapiServiceExtension;

    @Autowired
    private IEtOrderService etOrderService;

    @Autowired
    private IAsUserService asUserService;

//    @Autowired
//    private NotificationParser notificationParser;

//    @Autowired
//    public RefundService refundService2;

//    @Autowired
//    public ProfitsharingService profitsharingService;

    @Autowired
    private RedisLock redisLock;

    @Resource
    private EtAreaDeptMapper etAreaDeptMapper;

    @Autowired
    private ISysDeptService deptService;

    private static final String CNY = "CNY";

    private static final String PREPAY_LOCK = "prepay:";

    @Override
    public PrepayWithRequestPaymentResponse prepayWithRequestPayment(String payType,EtOrder order) {
        String billNo = order.getOrderNo();
        // 创建订单
        AsUser user = asUserService.selectUserById(order.getUserId());
        try {
            List<EtAreaDept> areaId1 = etAreaDeptMapper.selectList(new QueryWrapper<EtAreaDept>().eq("area_id", order.getAreaId()));
            if (ObjectUtil.isNotEmpty(areaId1) && areaId1.size() > 0){
                EtAreaDept etAreaDept = areaId1.get(0);
                SysDept sysDept = deptService.selectDeptById(etAreaDept.getDeptId());
                log.info("获取到运营商对象：【{}】",JSON.toJSON(sysDept));
                if(ObjectUtil.isNull(sysDept)){
                    throw new ServiceException("没有运营商：【"+etAreaDept.getDeptId()+"】");
                }

                String isProfitSharing = sysDept.getIsProfitSharing();
                // 获取JSAPI所需参数
                PrepayRequest request = new PrepayRequest();
                request.setAmount(getAmount(order.getPayFee()));
                String outTradeNo = IdUtils.getOrderNo("wx");
                order.setOutTradeNo(outTradeNo);
                int updateEtOrder = etOrderService.updateEtOrder(order);
                if(updateEtOrder == 0){
                    throw new ServiceException("更新订单outTradeNo失败");
                }
                request.setOutTradeNo(outTradeNo);
                request.setAppid(sysDept.getAppid());
                request.setMchid(sysDept.getMerchantId());
                String type = order.getType();
                String description = type.equals(ServiceConstants.ORDER_TYPE_RIDING) ? "骑行订单-"+billNo : "押金充值-"+billNo;
//                LoginUser loginUser = SecurityUtils.getLoginUser();
//                log.info("【预下单】获取登录用户信息:"+JSON.toJSONString(loginUser));
                request.setAttach(JSON.toJSONString(new AttachVo(payType,user.getUserId(), "")));
                request.setDescription(description);
                request.setNotifyUrl(sysDept.getNotifyUrl());
                request.setPayer(getPayer(user.getWxopenid()));
                SettleInfo settleInfo = new SettleInfo();
                settleInfo.setProfitSharing( "true".equalsIgnoreCase(isProfitSharing));
                request.setSettleInfo(settleInfo);
                JsapiServiceExtension jsapiServiceExtension = getJsapiServiceExtension(sysDept);
                PrepayWithRequestPaymentResponse res = jsapiServiceExtension.prepayWithRequestPayment(request);
                return res;
            }else{
                log.info("区域：【{}】没有绑定运营商",order.getAreaId());
                throw new ServiceException("区域：【"+order.getAreaId()+"】没有绑定运营商");
            }
        } finally {
            redisLock.unlock(PREPAY_LOCK + billNo);
        }
    }

    private JsapiServiceExtension getJsapiServiceExtension(SysDept sysDept) {
        // 初始化商户配置
        Config config = new RSAAutoCertificateConfig.Builder()
                .merchantId(sysDept.getMerchantId())
                // 使用 com.wechat.pay.java.core.util 中的函数从本地文件中加载商户私钥，商户私钥会用来生成请求的签名
                .privateKeyFromPath(sysDept.getPrivateKeyPath())
                .merchantSerialNumber(sysDept.getMerchantSerialNumber())
                .apiV3Key(sysDept.getApiV3Key())
                .build();
        // 初始化服务
        return new JsapiServiceExtension
                .Builder()
                .config(config)
                .signType("RSA") // 不填默认为RSA
                .build();
    }

    /**
     * 根据订单号查询订单
     * @param outTradeNo 商户订单号
     * @return
     */
    @Override
    public Transaction queryOrderByOutTradeNo(String outTradeNo) {
        EtOrder order = etOrderService.selectEtOrderByOutTradeNo(outTradeNo);
        List<EtAreaDept> areaId1 = etAreaDeptMapper.selectList(new QueryWrapper<EtAreaDept>().eq("area_id", order.getAreaId()));
        if (ObjectUtil.isNotEmpty(areaId1) && areaId1.size() > 0){
            EtAreaDept etAreaDept = areaId1.get(0);
            SysDept sysDept = deptService.selectDeptById(etAreaDept.getDeptId());
            log.info("获取到运营商对象：【{}】",JSON.toJSON(sysDept));
            QueryOrderByOutTradeNoRequest request = new QueryOrderByOutTradeNoRequest();
            request.setMchid(sysDept.getMerchantId());
            request.setOutTradeNo(outTradeNo);
            JsapiService jsapiService = getJsapiService(sysDept);
            return jsapiService.queryOrderByOutTradeNo(request);
        }else{
            log.info("区域：【{}】没有绑定运营商",order.getAreaId());
            throw new ServiceException("区域：【"+order.getAreaId()+"】没有绑定运营商");
        }
    }

    private JsapiService getJsapiService(SysDept sysDept) {
        // 初始化商户配置
        Config config = new RSAAutoCertificateConfig.Builder()
                .merchantId(sysDept.getMerchantId())
                // 使用 com.wechat.pay.java.core.util 中的函数从本地文件中加载商户私钥，商户私钥会用来生成请求的签名
                .privateKeyFromPath(sysDept.getPrivateKeyPath())
                .merchantSerialNumber(sysDept.getMerchantSerialNumber())
                .apiV3Key(sysDept.getApiV3Key())
                .build();
        // 初始化服务
        return new JsapiService.Builder().config(config).build();
    }

    /**
     * 退款
     * @param etOrder 原订单对象
     * @param reason 退款原因
     * @param amount 退款金额
     * @param outRefundNo 退款订单号
     */
    @Override
    public Refund refund(EtOrder etOrder,String reason,BigDecimal amount,String outRefundNo) {
        SysDept sysDept = getDeptObjByAreaId(etOrder.getAreaId());
        CreateRequest request = new CreateRequest();
        request.setOutTradeNo(etOrder.getOutTradeNo());
        request.setOutRefundNo(outRefundNo);
        request.setReason(reason);
        request.setAmount(getAmountReq(etOrder.getTotalFee(),amount));
        request.setNotifyUrl(sysDept.getRefundNotifyUrl());
        log.info("【退款】请求微信参数:【{}】",JSON.toJSONString(request));
        RefundService refundService = getRefundService(sysDept);
        Refund refund = refundService.create(request);
        log.info("【退款】微信返回结果:【{}】",JSON.toJSONString(refund));
        return refund;
    }

    private RefundService getRefundService(SysDept sysDept){
        // 初始化商户配置
        Config config = new RSAAutoCertificateConfig.Builder()
                .merchantId(sysDept.getMerchantId())
                // 使用 com.wechat.pay.java.core.util 中的函数从本地文件中加载商户私钥，商户私钥会用来生成请求的签名
                .privateKeyFromPath(sysDept.getPrivateKeyPath())
                .merchantSerialNumber(sysDept.getMerchantSerialNumber())
                .apiV3Key(sysDept.getApiV3Key())
                .build();
        // 初始化服务
        return new RefundService.Builder().config(config).build();
    }

    /**
     * 发起商家转账
     * @param sysDept 运营商对象
     * @param batchNo 批次号
     * @param batchName 批次名称
     * @param batchRemark 转账说明
     * @param totalAmount 转账总金额
     * @param totalNum 转账总笔数
     * @param transferDetailInputs 转账明细列表
     */
    @Override
    public InitiateBatchTransferResponse transfer(SysDept sysDept,String batchNo,String batchName,String batchRemark,BigDecimal totalAmount,Integer totalNum,List<TransferDetailInput> transferDetailInputs) {
//        SysDept sysDept = getDeptObjByAreaId(areaId);
        InitiateBatchTransferRequest request = new InitiateBatchTransferRequest();
        request.setAppid(sysDept.getAppid());
        request.setOutBatchNo(batchNo);
        request.setBatchName(batchName);
        request.setBatchRemark(batchRemark);
        request.setTotalAmount(totalAmount.multiply(new BigDecimal(100)).longValue());
        request.setTotalNum(totalNum);
        request.setTransferDetailList(transferDetailInputs);
        log.info("【发起商家转账】请求微信参数:【{}】",JSON.toJSONString(request));
        TransferBatchService transferService = getTransferService(sysDept);
        InitiateBatchTransferResponse initiateBatchTransferResponse = transferService.initiateBatchTransfer(request);
        log.info("【发起商家转账】微信返回结果:【{}】",JSON.toJSONString(initiateBatchTransferResponse));
        return initiateBatchTransferResponse;
    }

    private TransferBatchService getTransferService(SysDept sysDept){
        // 初始化商户配置
        Config config = new RSAAutoCertificateConfig.Builder()
                .merchantId(sysDept.getMerchantId())
                // 使用 com.wechat.pay.java.core.util 中的函数从本地文件中加载商户私钥，商户私钥会用来生成请求的签名
                .privateKeyFromPath(sysDept.getPrivateKeyPath())
                .merchantSerialNumber(sysDept.getMerchantSerialNumber())
                .apiV3Key(sysDept.getApiV3Key())
                .build();
        // 初始化服务
        return new TransferBatchService.Builder().config(config).build();
    }


    /**
     * 根据运营区id获取运营商对象
     * @param areaId
     * @return
     */
    @Override
    public SysDept getDeptObjByAreaId(Long areaId) {
        SysDept sysDept;
        List<EtAreaDept> areaId1 = etAreaDeptMapper.selectList(new QueryWrapper<EtAreaDept>().eq("area_id", areaId));
        if (ObjectUtil.isNotEmpty(areaId1) && areaId1.size() > 0){
            EtAreaDept etAreaDept = areaId1.get(0);
             sysDept = deptService.selectDeptById(etAreaDept.getDeptId());
            log.info("获取到运营商对象：【{}】",JSON.toJSON(sysDept));
        }else{
            log.info("区域：【{}】没有绑定运营商",areaId);
            throw new ServiceException("区域：【"+areaId+"】没有绑定运营商");
        }
        return sysDept;
    }

    @Override
    public Refund queryByOutRefundNo(Long areaId,String outRefundNo) {
        SysDept sysDept = getDeptObjByAreaId(areaId);
        QueryByOutRefundNoRequest request = new QueryByOutRefundNoRequest();
        request.setOutRefundNo(outRefundNo);
        RefundService refundService = getRefundService(sysDept);
        return refundService.queryByOutRefundNo(request);
    }

    /** 请求分账API */
    public OrdersEntity createOrder(SysDept sysDept,String transactionId,List<CreateOrderReceiver> receivers) {
        CreateOrderRequest request = new CreateOrderRequest();
        request.setAppid(sysDept.getAppid());
        request.setTransactionId(transactionId);// 微信订单号
        request.setOutOrderNo(IdUtils.getOrderNo("fz"));// 商户系统内部分账单号
        request.setReceivers(receivers);
        request.setUnfreezeUnsplit(true);
        ProfitsharingService profitsharingService = getProfitsharingService(sysDept);
        return profitsharingService.createOrder(request);
    }

    /** 添加分账接收方 */
    @Override
    public AddReceiverResponse addReceiver(String wxopenid,Long deptId,String type) {
        SysDept sysDept = deptService.selectDeptById(deptId);
        AddReceiverRequest request = new AddReceiverRequest();
        request.setAppid(sysDept.getAppid());
        if(type.equals(ServiceConstants.PROFITSHARING_TYPE_PLATFORM)){
            request.setType(ReceiverType.MERCHANT_ID);
        }else{
            request.setType(ReceiverType.PERSONAL_OPENID);
        }
        request.setAccount(wxopenid);
        request.setRelationType(ReceiverRelationType.PARTNER);
        ProfitsharingService profitsharingService = getProfitsharingService(sysDept);
        return profitsharingService.addReceiver(request);
    }

    /** 删除分账接收方 */
    public DeleteReceiverResponse deleteReceiver(String wxopenid,Long deptId,String type) {
        SysDept sysDept = deptService.selectDeptById(deptId);
        DeleteReceiverRequest request = new DeleteReceiverRequest();
        request.setAppid(sysDept.getAppid());
        if(type.equals(ServiceConstants.PROFITSHARING_TYPE_PLATFORM)){
            request.setType(ReceiverType.MERCHANT_ID);
        }else{
            request.setType(ReceiverType.PERSONAL_OPENID);
        }
        request.setAccount(wxopenid);
        ProfitsharingService profitsharingService = getProfitsharingService(sysDept);
        return profitsharingService.deleteReceiver(request);
    }

    private ProfitsharingService getProfitsharingService(SysDept sysDept) {
        Config config = new RSAAutoCertificateConfig.Builder()
                        .merchantId(sysDept.getMerchantId())
                        // 使用 com.wechat.pay.java.core.util中的函数从本地文件中加载商户私钥，商户私钥会用来生成请求的签名
                        .privateKeyFromPath(sysDept.getPrivateKeyPath())
                        .merchantSerialNumber(sysDept.getMerchantSerialNumber())
                        .apiV3Key(sysDept.getApiV3Key())
                        .build();
        // 初始化服务
        return new ProfitsharingService.Builder().config(config).build();
    }


//    /**
//     * 验签并解析
//     * @param request 请求
//     * @param body 请求体
//     * @param clazz 返回值类型
//     */
//    private <T> T checkAndParse(HttpServletRequest request, String body, Class<T> clazz) {
//        // 构造 RequestParam
//        RequestParam requestParam = new RequestParam.Builder()
//                .serialNumber(request.getHeader("Wechatpay-Serial"))
//                .nonce(request.getHeader("Wechatpay-Nonce"))
//                .signature(request.getHeader("Wechatpay-Signature"))
//                .timestamp(request.getHeader("Wechatpay-Timestamp"))
//                .body(body)
//                .build();
//        // 验签
//        return notificationParser.parse(requestParam, clazz);
//    }

    private Payer getPayer(String openId) {
        Payer payer = new Payer();
        payer.setOpenid(openId);
        return payer;
    }

    private Amount getAmount(BigDecimal money) {
        Amount amount = new Amount();
        amount.setTotal(money.multiply(new BigDecimal(100)).intValue());
        amount.setCurrency(CNY);
        return amount;
    }

    private AmountReq getAmountReq(BigDecimal total,BigDecimal rerundAmount) {
        AmountReq amount = new AmountReq();
        amount.setTotal(total.multiply(new BigDecimal(100)).longValue());
        amount.setRefund(rerundAmount.multiply(new BigDecimal(100)).longValue());
        amount.setCurrency(CNY);
        return amount;
    }


}
